package com.unissoft.utils;

import org.apache.commons.lang3.time.DateUtils;

import java.text.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.WeekFields;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import static cn.hutool.core.date.DateUtil.format;

/**
 * 时间工具类
 */
public class DateUtil {

    public static final String PATTERN_DATETIME = "yyyy-MM-dd HH:mm:ss";
    public static final String PATTERN_DATE = "yyyy-MM-dd";
    public static final String PATTERN_TIME = "HH:mm:ss";

    /**
     * yyyy-MM-dd HH:mm:ss转date类型
     */
    public static Date formatDate(String stringDate, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = null;
        try {
            date = sdf.parse(stringDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }


    /**
     * yyyy-MM-dd HH:mm:ss转date类型
     */
    public static Date formatDate(String stringDate) {
        SimpleDateFormat sdf = new SimpleDateFormat(PATTERN_DATETIME);
        Date date = null;
        try {
            date = sdf.parse(stringDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * date类型转yyyy-MM-dd HH:mm:ss
     */
    public static String formatDate(Date date) {
        Format format = new SimpleDateFormat(PATTERN_DATETIME);
        String stringDate = format.format(date);
        return stringDate;
    }

    /**
     * "2018-03-01T16:00:00.000Z"格式转换成date类型。
     */
    public static Date formatDate_Z(String stringDate) {
        //注意是空格+UTC
        stringDate = stringDate.replace("Z", " UTC");
        //注意格式化的表达式
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS Z");
        Date date = null;
        try {
            date = format.parse(stringDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 时间相加减转yyyy-MM-dd HH:mm:ss
     *
     * @param date
     * @param str  x年 或者x个月 或者x日/天
     * @return
     */
    public static String addTime(Date date, String str) {
        int size = 0;
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);//设置起时间
        if (str.contains("年")) {
            size = Integer.parseInt(str.substring(0, str.length() - 1));
            cal.add(Calendar.YEAR, size);//增加年
        }
        if (str.contains("个月")) {
            size = Integer.parseInt(str.substring(0, str.length() - 2));
            cal.add(Calendar.MONTH, size);//增加月
        }
        if (str.contains("日") || str.contains("天")) {
            size = Integer.parseInt(str.substring(0, str.length() - 1));
            cal.add(Calendar.DATE, size);//增加日/天
        }
        date = cal.getTime();
        Format format = new SimpleDateFormat(PATTERN_DATETIME);
        return format.format(date);
    }

    /**
     * date2比date1多的天数
     *
     * @param date1
     * @param date2
     * @return
     */
    public static int differentDays(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        if (year1 != year2)   //同一年
        {
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0)    //闰年
                {
                    timeDistance += 366;
                } else    //不是闰年
                {
                    timeDistance += 365;
                }
            }

            return timeDistance + (day2 - day1);
        } else    //不同年
        {
            System.err.println("判断day2 - day1 : " + (day2 - day1));
            return day2 - day1;
        }
    }

    /**
     * date2比date1多的秒数
     *
     * @param date1
     * @param date2
     * @return
     */
    public static int differentSecond(Date date1, Date date2) {
        long time = date2.getTime() - date1.getTime();
        int second = (int) time / 1000;
        return second;
    }

    /**
     * 获取具体指定的时间
     *
     * @param date
     * @param hour
     * @param minute
     * @param second
     * @return
     */
    public static Date getSpecificDate(Date date, Integer hour, Integer minute, Integer second) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.setLenient(true);

        if (null != hour) {
            c.set(Calendar.HOUR_OF_DAY, hour);
        }
        if (null != minute) {
            c.set(Calendar.MINUTE, minute);
        }
        if (null != second) {
            c.set(Calendar.SECOND, second);
        }
        c.set(Calendar.MILLISECOND, 0);
        return c.getTime();
    }

    /**
     * 获取当前时间0秒的日期
     *
     * @return
     */
    public static Date getCurrMinute() {
        Date date = new Date();
        return getSpecificDate(date, null, null, 0);
    }

    /**
     * 获取当前时间0分0秒的日期
     *
     * @return
     */
    public static Date getCurrHour() {
        Date date = new Date();
        return getSpecificDate(date, null, 0, 0);
    }


    /**
     * 获取当前日期 0时0分0秒 的日期
     *
     * @return
     */
    public static Date getCurrDate() {
        return getCurrDate(0, 0, 0);
    }

    /**
     * 获取当前日期 hour时minute分second秒 的日期
     *
     * @return
     */
    public static Date getCurrDate(Integer hour, Integer minute, Integer second) {
        Date date = new Date();
        return getSpecificDate(date, hour, minute, second);
    }

    /**
     * 获取当前周星期一 0时0分0秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrWeek() {
        return getFirstDateOfCurrWeek(0, 0, 0);
    }

    /**
     * 获取当前周星期一 hour时minute分second秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrWeek(int hour, int minute, int second) {
        Date date = new Date();

        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.setLenient(true);

        c.set(Calendar.DAY_OF_WEEK, 2);//星期一：2
        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        c.set(Calendar.MILLISECOND, 0);

        return c.getTime();
    }

    /**
     * 获取当前月第一天 0时0分0秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrMonth() {
        return getFirstDateOfCurrMonth(0, 0, 0);
    }

    /**
     * 获取当前月第一天 hour时minute分second秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrMonth(int hour, int minute, int second) {
        Date date = new Date();

        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.setLenient(true);

        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        c.set(Calendar.MILLISECOND, 0);

        return c.getTime();
    }

    /**
     * 获取当前季度第一天 0时0分0秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrSeason() {
        return getFirstDateOfCurrSeason(0, 0, 0);
    }

    /**
     * 获取当前季度第一天 hour时minute分second秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrSeason(int hour, int minute, int second) {
        Date date = new Date();

        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.setLenient(true);

        int month = c.get(Calendar.MONTH);
        c.set(Calendar.MONTH, month / 3 * 3);

        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        c.set(Calendar.MILLISECOND, 0);

        return c.getTime();
    }

    /**
     * 获取当前年第一天 0时0分0秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrYear() {
        return getFirstDateOfCurrYear(0, 0, 0);
    }

    /**
     * 获取当前年第一天 hour时minute分second秒 的日期
     *
     * @return
     */
    public static Date getFirstDateOfCurrYear(int hour, int minute, int second) {
        Date date = new Date();

        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.setLenient(true);

        c.set(Calendar.DAY_OF_YEAR, 1);
        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, second);
        c.set(Calendar.MILLISECOND, 0);

        return c.getTime();
    }


    /**
     * 日期提前几年
     *
     * @param date
     * @param years
     * @return
     */
    public static Date aheadOfYears(Date date, int years) {
        return DateUtils.addYears(date, 0 - years);
    }

    /**
     * 日期提前几个季度
     *
     * @param date
     * @param seasons
     * @return
     */
    public static Date aheadOfSeasons(Date date, int seasons) {
        return aheadOfMonths(date, 3 * seasons);
    }

    /**
     * 日期提前几个月
     *
     * @param date
     * @param months
     * @return
     */
    public static Date aheadOfMonths(Date date, int months) {
        return DateUtils.addMonths(date, 0 - months);
    }

    /**
     * 日期提前几周
     *
     * @param date
     * @param weeks
     * @return
     */
    public static Date aheadOfWeeks(Date date, int weeks) {
        return DateUtils.addWeeks(date, 0 - weeks);
    }

    /**
     * 日期提前几天
     *
     * @param date
     * @param days
     * @return
     */
    public static Date aheadOfDays(Date date, int days) {
        return DateUtils.addDays(date, 0 - days);
    }

    /**
     * 日期提前几个小时
     *
     * @param date
     * @param hours
     * @return
     */
    public static Date aheadOfHours(Date date, int hours) {
        return DateUtils.addHours(date, 0 - hours);
    }

    /**
     * 日期提前几分钟
     *
     * @param date
     * @param minutes
     * @return
     */
    public static Date aheadOfMinutes(Date date, int minutes) {
        return DateUtils.addMinutes(date, 0 - minutes);
    }


    /**
     * 功能描述：日期转换cron表达式
     *
     * @param date
     * @param dateFormat
     * @return
     */
    public static String formatDateByPattern(Date date, String dateFormat) {
        String result = null;
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        if (date != null) {
            result = sdf.format(date);
        }
        return result;
    }

    /**
     * 日期转换cron表达式
     *
     * @param date
     * @return
     */
    public static String getCron(Date date) {
        String dateFormat = "ss mm HH dd MM ? yyyy";
        return formatDateByPattern(date, dateFormat);
    }

    /**
     * 格式化获取年
     *
     * @param date    时间
     * @param pattern 格式类型 (yyyy-MM-dd HH:mm:ss)
     * @return
     */
    public static String getYMDHMS(Date date, String pattern) {
        return format(LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault()), pattern);
    }

    /**
     * 获取当前时间 为一年中的第多少周
     *
     * @param date
     * @return
     */
    public static Integer getPresentWeek(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
        calendar.setTime(date);
        return calendar.get(Calendar.WEEK_OF_YEAR);
    }

    /**
     * 获取当前时间
     *
     * @return
     * @throws ParseException
     */
    public static Date getTime() throws ParseException {
        Date time = Calendar.getInstance().getTime();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        String format = dateFormat.format(time);
        Date date = dateFormat.parse(format);
        return date;
    }

    /**
     * 获取当前时间年月
     *
     * @return
     * @throws ParseException
     */
    public static Date getSpecificTime() {
        Date date = null;
        try {
            Date time = Calendar.getInstance().getTime();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");
            String format = dateFormat.format(time);
            date = dateFormat.parse(format);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 获取当前时间年月日
     *
     * @return
     * @throws ParseException
     */
    public static Date getDayTime() {
        Date date = null;
        try {
            Date time = Calendar.getInstance().getTime();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String format = dateFormat.format(time);
            date = dateFormat.parse(format);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 获取上一个月的日期
     *
     * @return
     * @throws ParseException
     */
    public static String getLastMonth() {
        Calendar c = Calendar.getInstance();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        c.setTime(new Date());
        c.add(Calendar.MONTH, -1);
        Date m = c.getTime();
        String mon = dateFormat.format(m);
        return mon;
    }

    /**
     * 获取当月的 天数
     */
    public static int getCurrentMonthDay() {
        Calendar a = Calendar.getInstance();
        a.set(Calendar.DATE, 1);
        a.roll(Calendar.DATE, -1);
        int maxDate = a.get(Calendar.DATE);
        return maxDate;
    }
}
