package com.unissoft.webConfig;

import cn.hutool.json.JSONUtil;
import com.unissoft.MyConstant;
import com.unissoft.exception.StudentException;
import com.unissoft.result.ResultView;
import com.unissoft.webConfig.service.impl.AccessLimit;
import com.unissoft.webConfig.service.impl.AutoIdempotent;
import com.unissoft.webConfig.service.impl.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;

/**
 * 文件描述 拦截器
 *
 * @author tongxi.xia
 * @date 2021年02月05日 10:28
 */
@Component
public class AutoIdempotentInterceptor implements HandlerInterceptor {

    @Autowired
    private TokenService tokenService;


    /**
     * 预处理
     *
     * @param request
     * @param response
     * @param handler
     * @return
     * @throws Exception
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        //被ApiIdempotment标记的扫描，幂等
        AutoIdempotent methodAnnotation = method.getAnnotation(AutoIdempotent.class);
        if (methodAnnotation != null) {
            try {
                // 幂等性校验, 校验通过则放行, 校验失败则抛出异常, 并通过统一异常处理返回友好提示
                return tokenService.checkToken(request);
            } catch (StudentException ex) {
                ResultView failedResult = ResultView.error(ex.toString());

                writeReturnJson(response, JSONUtil.toJsonStr(failedResult));
                throw ex;
            }
        }

        //限制请求次数
        AccessLimit accessLimit = method.getAnnotation(AccessLimit.class);
        if (accessLimit != null) {
            boolean accessLimitRet = tokenService.getAccessLimit(request, response, accessLimit);
            return accessLimitRet;
        }
        //必须返回true,否则会被拦截一切请求
        return true;
    }


    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws StudentException {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws StudentException {

    }

    /**
     * 返回的json值
     *
     * @param response
     * @param json
     * @throws Exception
     */
    private void writeReturnJson(HttpServletResponse response, String json) throws StudentException {
        PrintWriter writer = null;
        response.setCharacterEncoding(MyConstant.CHARACTER_ENCODING);
        response.setContentType(MyConstant.CONTENT_TYPE);
        try {
            writer = response.getWriter();
            writer.print(json);

        } catch (IOException e) {
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }
}
